import hashlib
import hmac
import logging
import time
from urllib.parse import quote, urlencode

from requests.auth import AuthBase

logger = logging.getLogger(__name__)


def to_bytes(v):
    if isinstance(v, str):
        return v.encode('utf-8')
    return v


def filter_headers(data):
    """只设置host content-type 还有x开头的头部.

    :param data(dict): 所有的头部信息.
    :return(dict): 计算进签名的头部.
    """
    headers = {}
    for i in data:
        if i == 'Content-Type' or i == 'Host' or i[0] == 'x' or i[0] == 'X':
            headers[i] = data[i]
    return headers


class CosS3Auth(AuthBase):

    def __init__(self, secret_id, secret_key, key=None, params={}, expire=10000):
        self._secret_id = secret_id
        self._secret_key = secret_key
        self._expire = expire
        self._params = params
        if key:
            if key[0] == u'/':
                self._path = key
            else:
                self._path = u'/' + key
        else:
            self._path = u'/'

    def __call__(self, r):
        path = self._path
        uri_params = self._params
        headers = filter_headers(r.headers)
        uri_params = dict([(k.lower(), v) for k, v in uri_params.items()])
        # reserved keywords in headers urlencode are -_.~,
        # notice that / should be encoded and space should not be encoded to plus sign(+)

        # headers中的key转换为小写，value进行encode
        headers = dict([(k.lower(), quote(to_bytes(v), '-_.~')) for k, v in headers.items()])
        uri_params = dict([(k.lower(), v) for k, v in uri_params.items()])
        format_str = u"{method}\n{host}\n{params}\n{headers}\n".format(
            method=r.method.lower(),
            host=path,
            params=urlencode(sorted(uri_params.items())).replace('+', '%20').replace('%7E', '~'),
            headers='&'.join(map(lambda tupl: "%s=%s" % (tupl[0], tupl[1]), sorted(headers.items())))
        )
        logger.debug("format str: " + format_str)

        start_sign_time = int(time.time())
        sign_time = "{bg_time};{ed_time}".format(bg_time=start_sign_time-60, ed_time=start_sign_time+self._expire)
        sha1 = hashlib.sha1()
        sha1.update(to_bytes(format_str))

        str_to_sign = "sha1\n{time}\n{sha1}\n".format(time=sign_time, sha1=sha1.hexdigest())
        logger.debug('str_to_sign: ' + str(str_to_sign))
        sign_key = hmac.new(to_bytes(self._secret_key), to_bytes(sign_time), hashlib.sha1).hexdigest()
        sign = hmac.new(to_bytes(sign_key), to_bytes(str_to_sign), hashlib.sha1).hexdigest()
        logger.debug('sign_key: ' + str(sign_key))
        logger.debug('sign: ' + str(sign))
        sign_tpl = ("q-sign-algorithm=sha1&q-ak={ak}&q-sign-time={sign_time}&q-key-time={key_time}"
                    "&q-header-list={headers}&q-url-param-list={params}&q-signature={sign}")

        r.headers['Authorization'] = sign_tpl.format(
            ak=self._secret_id,
            sign_time=sign_time,
            key_time=sign_time,
            params=';'.join(sorted(map(lambda k: k.lower(), uri_params.keys()))),
            headers=';'.join(sorted(headers.keys())),
            sign=sign
        )
        logger.debug("sign_key" + str(sign_key))
        logger.debug(r.headers['Authorization'])
        logger.debug("request headers: " + str(r.headers))
        return r


if __name__ == "__main__":
    pass



import gzip
import json
import mimetypes
from urllib.parse import urljoin

import requests

from .auth import CosS3Auth


class Client:
    def __init__(self, secret_id, secret_key, region, bucket, scheme="https"):
        assert scheme in ["http", "https"]

        self.secret_id = secret_id
        self.secret_key = secret_key

        self.region = region
        self.bucket = bucket
        self.scheme = scheme

    def get_object_url(self, key):
        return urljoin(
            f"{self.scheme}://{self.bucket}.cos.{self.region}.myqcloud.com", key
        )

    def request(self, method, key, **kwargs):
        url = self.get_object_url(key)
        return requests.request(
            method, url, auth=CosS3Auth(self.secret_id, self.secret_key, key), **kwargs,
        )

    def head_object(self, key):
        return self.request("HEAD", key)

    def get_object(self, key):
        return self.request("GET", key)

    def put_object(self, key, data, content_type=None, headers={}, **kwargs):
        if content_type is None:
            content_type = guess_content_type(key)
        headers["Content-Type"] = content_type
        return self.request("PUT", key, data=data, headers=headers, **kwargs)

    def put_local(self, key, local_path, **kwargs):
        with open(local_path, "rb") as f:
            return self.request("PUT", key, data=f, **kwargs)

    def smart_put_object(self, key, data, content_type=None, headers={}, **kwargs):
        if isinstance(data, (dict, list)):
            content_type = "application/json"
            data = json.dumps(data)
        if isinstance(data, str):
            data = data.encode("utf-8")
        gzip_data = gzip.compress(data)
        if len(data) > len(gzip_data):
            headers["Content-Encoding"] = "gzip"
            data = gzip_data
        return self.put_object(key, data, content_type, headers=headers, **kwargs)

    def get_object_or_none(self, key):
        """如果status_code是404，返回None"""
        r = self.get_object(key)
        if r.status_code == 404:
            return
        return r


def guess_content_type(filename):
    content_type = mimetypes.guess_type(filename)[0]
    if not content_type:
        return "application/octet-stream"
    return content_type



import argparse
import os.path

from termcolor import cprint

from . import Client, Configure

parser = argparse.ArgumentParser(description='腾讯云COS管理.')
parser.add_argument('local_dir')
parser.add_argument('region')
parser.add_argument('bucket')
parser.add_argument('cos_dir')


def main():
    # 参数包含local_path, bucket
    parsed_args = parser.parse_args()

    # 命令行参数
    local_dir = parsed_args.local_dir
    region = parsed_args.region
    bucket = parsed_args.bucket
    cos_dir = parsed_args.cos_dir

    local_dir = os.path.abspath(os.path.expanduser(local_dir))
    slice_start = len(local_dir) + 1

    print('local_dir: ', local_dir)
    print('bucket: ', bucket, 'cos_dir: ', cos_dir)

    # 配置文件配置
    config = Configure()
    client = Client(config.secret_id, config.secret_key, region, bucket)

    for root, dirs, files in os.walk(local_dir):
        for f in files:
            local_path = os.path.join(root, f)
            key = os.path.join(cos_dir, local_path[slice_start:])
            if local_path.endswith('.html') or client.head_object(key).status_code == 404:
                r = client.put_local(key, local_path)
                cprint(f'已上传 {key} {r.status_code}', 'blue')
                assert r.status_code == 200
            else:
                cprint(f'已存在 {key}', 'red')



#!/usr/bin/env python
from setuptools import find_packages, setup

with open("README.rst") as f:
    readme = f.read()


setup(
    name="qcos",
    version="2.1.5",
    description="腾讯云对象存储库，支持命令行",
    long_description=readme,
    author="codeif",
    author_email="me@codeif.com",
    url="https://github.com/codeif/qcos",
    license="MIT",
    entry_points={"console_scripts": ["qcos = qcos.cli:main"]},
    install_requires=["requests", "termcolor"],
    packages=find_packages(exclude=("tests", "tests.*")),
)



import json
from pathlib import Path


class Configure(object):
    """加载tccli配置的配置文件
    https://cloud.tencent.com/document/product/440
    """
    def __init__(self):
        conf = self._load_json('default.configure')
        cred = self._load_json('default.credential')

        self.region = conf.get('region')
        self.secret_id = cred.get('secretId')
        self.secret_key = cred.get('secretKey')
        assert self.region and self.secret_id

    def _load_json(self, name):
        p = Path.home() / '.tccli' / name
        if not p.exists():
            return
        with p.open() as f:
            return json.load(f)
